from ._color import Color, reset


def ansi_print(text="", *args, color: Color = "", sep: str = ' ', end: str = f"{Color(reset.all)}\n", file=None):
    print(f"{color}{text}", *args, sep=sep, end=end, file=file)


def ansi_string(text="", *args, color: Color = "", sep: str = ' ', end: str = f"{Color(reset.all)}\n"):
    return f"{color}{sep.join([text] + [str(arg) for arg in args])}{end}"




import unittest
from ansi_terminal import ansi_print, ansi_string, Color, fg, bg, style, reset, Display, erase, Cursor, move


class Test(unittest.TestCase):
    def test_color_foreground_names(self):
        color_list = (
            ("black", Color(fg.black)),
            ("red", Color(fg.red)),
            ("green", Color(fg.green)),
            ("yellow", Color(fg.yellow)),
            ("blue", Color(fg.blue)),
            ("magenta", Color(fg.magenta)),
            ("cyan", Color(fg.cyan)),
            ("white", Color(fg.white)),
            ("", Color()),
            ("bright_black", Color(fg.bright_black)),
            ("bright_red", Color(fg.bright_red)),
            ("bright_green", Color(fg.bright_green)),
            ("bright_yellow", Color(fg.bright_yellow)),
            ("bright_blue", Color(fg.bright_blue)),
            ("bright_magenta", Color(fg.bright_magenta)),
            ("bright_cyan", Color(fg.bright_cyan)),
            ("bright_white", Color(fg.bright_white)),
            ("", Color())
        )
        for color in color_list:
            if color[0] == "":
                ansi_print()
            else:
                ansi_print(f"{color[0]:18}", color=color[1], end="")

    def test_color_background_names(self):
        color_list = (
            ("black", Color(bg.black)),
            ("red", Color(bg.red)),
            ("green", Color(bg.green)),
            ("yellow", Color(bg.yellow)),
            ("blue", Color(bg.blue)),
            ("magenta", Color(bg.magenta)),
            ("cyan", Color(bg.cyan)),
            ("white", Color(bg.white)),
            ("", Color()),
            ("bright_black", Color(bg.bright_black)),
            ("bright_red", Color(bg.bright_red)),
            ("bright_green", Color(bg.bright_green)),
            ("bright_yellow", Color(bg.bright_yellow)),
            ("bright_blue", Color(bg.bright_blue)),
            ("bright_magenta", Color(bg.bright_magenta)),
            ("bright_cyan", Color(bg.bright_cyan)),
            ("bright_white", Color(bg.bright_white)),
            ("", Color())
        )
        for color in color_list:
            if color[0] == "":
                ansi_print()
            else:
                ansi_print(f"{color[0]:18}", color=color[1], end="")

    def test_color_id(self):
        for color_id in range(0, 16):
            ansi_print(f"{color_id:18}", color=Color(bg.id(color_id)), end=f"")
            if color_id in [7, 15, 51, 87, 123, 159, 195, 231, 255]:
                ansi_print()

        for color_id in range(16, 232):
            ansi_print(f"{color_id:4}", color=Color(bg.id(color_id)), end=f"")
            if color_id in [7, 15, 51, 87, 123, 159, 195, 231, 255]:
                ansi_print()

        for color_id in range(232, 256):
            ansi_print(f"{color_id:6}", color=Color(bg.id(color_id)), end=f"")
            if color_id in [7, 15, 51, 87, 123, 159, 195, 231, 255]:
                ansi_print()

    def test_color_rgb(self):
        for color_r in range(0, 256, 32):
            for color_g in range(0, 256, 48):
                for color_b in range(0, 256, 11):
                    ansi_print(f" ", color=Color(bg.rgb(color_r, color_g, color_b)), end=f"")
            ansi_print()

    def test_style(self):
        color_list = {
            "bold": Color(style.bold),
            "faint": Color(style.faint),
            "italic": Color(style.italic),
            "underline": Color(style.underline),
            "blink_slow": Color(style.blink_slow),
            "blink_fast": Color(style.blink_fast),
            "invert_color": Color(style.invert_color),
            "hide": Color(style.hide),
            "strike": Color(style.strike),
            "reset_font": Color(style.reset_font),
            "font_1": Color(style.font_1),
            "font_2": Color(style.font_2),
            "font_3": Color(style.font_3),
            "font_4": Color(style.font_4),
            "font_5": Color(style.font_5),
            "font_6": Color(style.font_6),
            "font_7": Color(style.font_7),
            "font_8": Color(style.font_8),
            "font_9": Color(style.font_9),
            "gothic": Color(style.gothic),
            "underline_double": Color(style.underline_double)
        }
        for color in color_list.items():
            ansi_print(color[0], color=color[1])


if __name__ == '__main__':
    unittest.main()




class style:
    bold = 1
    faint = 2
    italic = 3
    underline = 4
    blink_slow = 5
    blink_fast = 6
    invert_color = 7
    hide = 8
    strike = 9
    reset_font = 10
    font_1 = 11
    font_2 = 12
    font_3 = 13
    font_4 = 14
    font_5 = 15
    font_6 = 16
    font_7 = 17
    font_8 = 18
    font_9 = 19
    gothic = 20
    underline_double = 21




class fg:
    reset = 39

    black = 30
    red = 31
    green = 32
    yellow = 33
    blue = 34
    magenta = 35
    cyan = 36
    white = 37

    bright_black = 90
    bright_red = 91
    bright_green = 92
    bright_yellow = 93
    bright_blue = 94
    bright_magenta = 95
    bright_cyan = 96
    bright_white = 97

    @staticmethod
    def id(n: int):
        return f"38;5;{n}"

    @staticmethod
    def rgb(r: int, g: int, b: int):
        return f"38;2;{r};{g};{b}"




class move:
    home = "H"

    @staticmethod
    def position(line, column):
        return f"{line};{column}H"

    @staticmethod
    def up(n):
        return f"{n}A"

    @staticmethod
    def down(n):
        return f"{n}B"

    @staticmethod
    def right(n):
        return f"{n}C"

    @staticmethod
    def left(n):
        return f"{n}D"



from importlib.metadata import version
__version__ = version('ansi_terminal')

from ._ansi_print import ansi_print, ansi_string
from ._color import Color, bg, fg, style, reset
from ._cursor import Cursor, move
from ._display import Display, erase





class erase:
    in_display = "J"
    till_end_display = "0J"
    from_start_display = "1J"
    entire_display = "2J"
    saved = "3J"

    in_line = "K"
    till_end_line = "0K"
    from_start_line = "1K"
    entire_line = "2K"



from .._constants import ansi_escape

class Color:
    def __init__(self, *attributes):
        self.values = attributes

    def __str__(self):
        return f"{ansi_escape}[{';'.join(str(x) for x in self.values)}m"

    def __repr__(self):
        return f"{ansi_escape}[{';'.join(str(x) for x in self.values)}m"

    def __add__(self, other):
        return str(self) + str(other)

    def apply(self):
        print(str(self), end="")




class bg:
    reset = 49

    black = 40
    red = 41
    green = 42
    yellow = 43
    blue = 44
    magenta = 45
    cyan = 46
    white = 47

    bright_black = 100
    bright_red = 101
    bright_green = 102
    bright_yellow = 103
    bright_blue = 104
    bright_magenta = 105
    bright_cyan = 106
    bright_white = 107

    @staticmethod
    def id(n: int):
        return f"48;5;{n}"

    @staticmethod
    def rgb(r: int, g: int, b: int):
        return f"48;2;{r};{g};{b}"
